# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mslib/msui/ui/ui_wms_multilayers.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MultilayersDialog(object):
    def setupUi(self, MultilayersDialog):
        MultilayersDialog.setObjectName("MultilayersDialog")
        MultilayersDialog.resize(892, 396)
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(MultilayersDialog)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.cbMultilayering = QtWidgets.QCheckBox(MultilayersDialog)
        self.cbMultilayering.setObjectName("cbMultilayering")
        self.horizontalLayout_11.addWidget(self.cbMultilayering)
        self.line = QtWidgets.QFrame(MultilayersDialog)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.horizontalLayout_11.addWidget(self.line)
        self.lFilter = QtWidgets.QLabel(MultilayersDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lFilter.sizePolicy().hasHeightForWidth())
        self.lFilter.setSizePolicy(sizePolicy)
        self.lFilter.setObjectName("lFilter")
        self.horizontalLayout_11.addWidget(self.lFilter)
        self.leMultiFilter = QtWidgets.QLineEdit(MultilayersDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.leMultiFilter.sizePolicy().hasHeightForWidth())
        self.leMultiFilter.setSizePolicy(sizePolicy)
        self.leMultiFilter.setMaximumSize(QtCore.QSize(16777215, 20))
        self.leMultiFilter.setObjectName("leMultiFilter")
        self.horizontalLayout_11.addWidget(self.leMultiFilter)
        self.verticalLayout_3.addLayout(self.horizontalLayout_11)
        self.listLayers = QtWidgets.QTreeWidget(MultilayersDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listLayers.sizePolicy().hasHeightForWidth())
        self.listLayers.setSizePolicy(sizePolicy)
        self.listLayers.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.listLayers.setAutoFillBackground(False)
        self.listLayers.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.listLayers.setDragEnabled(False)
        self.listLayers.setDragDropOverwriteMode(False)
        self.listLayers.setAlternatingRowColors(True)
        self.listLayers.setAnimated(False)
        self.listLayers.setHeaderHidden(False)
        self.listLayers.setColumnCount(4)
        self.listLayers.setObjectName("listLayers")
        self.listLayers.headerItem().setText(0, "Name")
        self.listLayers.header().setVisible(True)
        self.listLayers.header().setCascadingSectionResizes(True)
        self.listLayers.header().setStretchLastSection(False)
        self.verticalLayout_3.addWidget(self.listLayers)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.label_7 = QtWidgets.QLabel(MultilayersDialog)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_12.addWidget(self.label_7)
        self.cbWMS_URL = QtWidgets.QComboBox(MultilayersDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cbWMS_URL.sizePolicy().hasHeightForWidth())
        self.cbWMS_URL.setSizePolicy(sizePolicy)
        self.cbWMS_URL.setStyleSheet("QComboBox QAbstractItemView { min-width: 800px; }")
        self.cbWMS_URL.setEditable(True)
        self.cbWMS_URL.setObjectName("cbWMS_URL")
        self.cbWMS_URL.addItem("")
        self.cbWMS_URL.addItem("")
        self.horizontalLayout_12.addWidget(self.cbWMS_URL)
        self.btGetCapabilities = QtWidgets.QPushButton(MultilayersDialog)
        self.btGetCapabilities.setObjectName("btGetCapabilities")
        self.horizontalLayout_12.addWidget(self.btGetCapabilities)
        self.pbViewCapabilities = QtWidgets.QPushButton(MultilayersDialog)
        self.pbViewCapabilities.setObjectName("pbViewCapabilities")
        self.horizontalLayout_12.addWidget(self.pbViewCapabilities)
        self.verticalLayout_3.addLayout(self.horizontalLayout_12)
        self.verticalLayout_4.addLayout(self.verticalLayout_3)
        self.actionCloseWindow = QtWidgets.QAction(MultilayersDialog)
        self.actionCloseWindow.setObjectName("actionCloseWindow")
        MultilayersDialog.addAction(self.actionCloseWindow)

        self.retranslateUi(MultilayersDialog)
        self.actionCloseWindow.triggered.connect(MultilayersDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(MultilayersDialog)

    def retranslateUi(self, MultilayersDialog):
        _translate = QtCore.QCoreApplication.translate
        MultilayersDialog.setWindowTitle(_translate("MultilayersDialog", "Layer List"))
        self.cbMultilayering.setText(_translate("MultilayersDialog", "Multilayering"))
        self.lFilter.setText(_translate("MultilayersDialog", "Layer Filter:"))
        self.listLayers.headerItem().setText(1, _translate("MultilayersDialog", "Style"))
        self.listLayers.headerItem().setText(2, _translate("MultilayersDialog", "Priority"))
        self.listLayers.headerItem().setText(3, _translate("MultilayersDialog", "Colour"))
        self.label_7.setText(_translate("MultilayersDialog", "WMS URL:"))
        self.cbWMS_URL.setToolTip(_translate("MultilayersDialog", "Enter a valid WMS URL here."))
        self.cbWMS_URL.setItemText(0, _translate("MultilayersDialog", "http://localhost:8002/fc_wms"))
        self.cbWMS_URL.setItemText(1, _translate("MultilayersDialog", "http://osm.omniscale.net/proxy/service"))
        self.btGetCapabilities.setToolTip(_translate("MultilayersDialog", "Request the capabilities from the WMS server."))
        self.btGetCapabilities.setText(_translate("MultilayersDialog", "get capabilities"))
        self.pbViewCapabilities.setToolTip(_translate("MultilayersDialog", "Show information on the selected WMS server."))
        self.pbViewCapabilities.setText(_translate("MultilayersDialog", "view"))
        self.actionCloseWindow.setText(_translate("MultilayersDialog", "Close Window"))
        self.actionCloseWindow.setShortcut(_translate("MultilayersDialog", "Ctrl+W"))
