<p>Welcome {{ username}} !</p>
<br>
<p>Thanks for signing up. Please follow this link to activate your account:</p>
<br>
<p><a href="{{ confirm_url }}">{{ confirm_url }}</a></p>
<p> Afterwards login by the user interface</p>
<br>
<p>Cheers!</p>